use axum::{Json, extract::Query};
use serde::{Deserialize, Serialize};

use crate::{HOST, USER};

#[derive(Deserialize)]
pub struct WebfingerQuery {
    resource: String,
}

#[derive(Serialize)]
pub struct WebfingerResponse {
    subject: String,
    alias: Vec<String>,
    links: Vec<WebfigerLink>,
}

#[derive(Serialize)]
pub struct WebfigerLink {
    rel: String,
    r#type: String,
    href: String,
}

pub async fn webfinger_get(
    Query(WebfingerQuery { resource }): Query<WebfingerQuery>,
) -> Json<WebfingerResponse> {
    let Some((typ, val)) = resource.split_once(':') else {
        panic!()
    };
    assert_eq!(typ, "acct");
    let Some((usr, dom)) = val.split_once('@') else {
        panic!()
    };
    assert_eq!(usr, USER);
    assert_eq!(dom, HOST);
    Json(WebfingerResponse {
        subject: format!("acct:{USER}@{HOST}"),
        alias: vec![format!("https://{HOST}/users/{USER}")],
        links: vec![WebfigerLink {
            rel: "self".to_string(),
            r#type: "application/activity+json".to_string(),
            href: format!("https://{HOST}/users/{USER}"),
        }],
    })
}
